'use strict';

exports.__esModule = true;
exports.default = createSpacing;

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var warnOnce = void 0;

function createSpacing() {
  var spacingInput = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 8;

  // Already transformed.
  if (spacingInput.mui) {
    return spacingInput;
  }

  // All components align to an 8dp square baseline grid for mobile, tablet, and desktop.
  // https://material.io/design/layout/understanding-layout.html#pixel-density
  var transform = void 0;

  if (typeof spacingInput === 'function') {
    transform = spacingInput;
  } else {
    (0, _warning2.default)(typeof spacingInput === 'number', ['Material-UI: the `theme.spacing` value (' + spacingInput + ') is invalid.', 'It should be a number or a function.'].join('\n'));
    transform = function transform(factor) {
      (0, _warning2.default)(typeof factor === 'number', 'Expected spacing argument to be a number, got ' + factor);
      return spacingInput * factor;
    };
  }

  var spacing = function spacing() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    (0, _warning2.default)(args.length <= 4, 'Material-UI: Too many arguments provided, expected between 0 and 4, got ' + args.length);

    if (args.length === 0) {
      return transform(1);
    }

    if (args.length === 1) {
      return transform(args[0]);
    }

    return args.map(function (factor) {
      var output = transform(factor);
      return typeof output === 'number' ? output + 'px' : output;
    }).join(' ');
  };

  // Backward compatibility, to remove in v5.
  Object.defineProperty(spacing, 'unit', {
    get: function get() {
      if (process.env.NODE_ENV !== 'production') {
        (0, _warning2.default)(warnOnce && process.env.NODE_ENV !== 'test', ['Material-UI: theme.spacing.unit usage has been deprecated.', 'It will be removed in v5.', 'You can replace `theme.spacing.unit * y` with `theme.spacing(y)`.', '', 'You can use the `https://github.com/mui-org/material-ui/tree/master/packages/material-ui-codemod/README.md#theme-spacing-api` migration helper to make the process smoother.'].join('\n'));
        warnOnce = true;
      }
      return spacingInput;
    }
  });
  spacing.mui = true;

  return spacing;
}
module.exports = exports['default'];